<?php
if (!defined('ABSPATH')) {
    exit;
}

function mrpu_sanitize_options( $input ) {
    return mrpu_Bulk_Usernames_Settings::sanitize_options( $input );
}

class mrpu_Bulk_Usernames_Settings {

    public function __construct() {
        add_action('admin_init', [$this, 'register_settings']);
    }

    public function register_settings() {
        register_setting(
            'mrpu_display_settings',
            'mrpu_display_options',
            array(
                // The third parameter accepts dynamic values (e.g., strings, callables, anonymous functions)
                // that PHPCS cannot reliably analyze for sanitization. Ensure data is sanitized before storage.
                'sanitize_callback' => 'mrpu_Bulk_Usernames_Settings::sanitize_options', // phpcs:ignore PluginCheck.CodeAnalysis.SettingSanitization.register_settingDynamic
            )
        );

        add_settings_section(
            'mrpu_display_section',
            '',
            [$this, 'display_section_callback'],
            'mrpu-bulk-usernames'
        );

        add_settings_field(
            'custom_text',
            __('Custom Uploader Text', 'my-random-post-usernames'),
            [$this, 'custom_text_callback'],
            'mrpu-bulk-usernames',
            'mrpu_display_section'
        );

        add_settings_field(
            'background_color',
            __('Background Color', 'my-random-post-usernames'),
            [$this, 'background_color_callback'],
            'mrpu-bulk-usernames',
            'mrpu_display_section'
        );

        add_settings_field(
            'text_color',
            __('Text Color', 'my-random-post-usernames'),
            [$this, 'text_color_callback'],
            'mrpu-bulk-usernames',
            'mrpu_display_section'
        );

        add_settings_field(
            'font_size',
            __('Font Size (px)', 'my-random-post-usernames'),
            [$this, 'font_size_callback'],
            'mrpu-bulk-usernames',
            'mrpu_display_section'
        );

        add_settings_field(
            'border_radius',
            __('Border Radius (px)', 'my-random-post-usernames'),
            [$this, 'border_radius_callback'],
            'mrpu-bulk-usernames',
            'mrpu_display_section'
        );
        
        add_settings_field(
            'username_color',
            __('Username Color', 'my-random-post-usernames'),
            [$this, 'username_color_callback'],
            'mrpu-bulk-usernames',
            'mrpu_display_section'
        );
        
        add_settings_field(
            'username_font_size',
            __('Username Font Size (px)', 'my-random-post-usernames'),
            [$this, 'username_font_size_callback'],
            'mrpu-bulk-usernames',
            'mrpu_display_section'
        );
        
        add_settings_field(
            'username_font_weight',
            __('Username Font Weight', 'my-random-post-usernames'),
            [$this, 'username_font_weight_callback'],
            'mrpu-bulk-usernames',
            'mrpu_display_section'
        );
        
        add_settings_field(
            'container_padding',
            __('Container Padding (px)', 'my-random-post-usernames'),
            [$this, 'container_padding_callback'],
            'mrpu-bulk-usernames',
            'mrpu_display_section'
        );
    }
    
    public static function sanitize_options($input) {
        $sanitized = array();

        // Sanitize text field
        if (isset($input['custom_text'])) {
            $sanitized['custom_text'] = sanitize_text_field($input['custom_text']);
        }
    
        // Sanitize color fields
        if (isset($input['background_color'])) {
            $sanitized['background_color'] = sanitize_hex_color($input['background_color']);
        }
    
        if (isset($input['text_color'])) {
            $sanitized['text_color'] = sanitize_hex_color($input['text_color']);
        }
    
        if (isset($input['username_color'])) {
            $sanitized['username_color'] = sanitize_hex_color($input['username_color']);
        }
    
        // Sanitize numeric fields
        $numeric_fields = array(
            'font_size',
            'border_radius',
            'username_font_size',
            'container_padding'
        );
    
        foreach ($numeric_fields as $field) {
            if (isset($input[$field])) {
                $sanitized[$field] = absint($input[$field]);
            }
        }
    
        // Sanitize font weight
        if (isset($input['username_font_weight'])) {
            $sanitized['username_font_weight'] = in_array($input['username_font_weight'], array('normal', 'bold', 'bolder', 'lighter'))
                ? $input['username_font_weight']
                : 'normal';
        }
    
        return $sanitized;
    }

    public function display_section_callback() {
        echo '<p>' . esc_html__('Customize how usernames appear on the frontend of your site.', 'my-random-post-usernames').'</p>';
    }

    public function custom_text_callback() {
        $options = get_option('mrpu_display_options');
        echo '<input type="text" name="mrpu_display_options[custom_text]" value="'.esc_attr($options['custom_text'] ?? 'Uploaded by').'" class="regular-text">';
    }

    public function background_color_callback() {
        $options = get_option('mrpu_display_options');
        echo '<input type="text" name="mrpu_display_options[background_color]" 
                    value="'.esc_attr($options['background_color'] ?? '#1b1b1b').'" 
                    class="mrpu-color-field"
                    data-preview-target="background-color">';
    }

    public function text_color_callback() {
        $options = get_option('mrpu_display_options');
        echo '<input type="text" name="mrpu_display_options[text_color]" 
                    value="'.esc_attr($options['text_color'] ?? '#b8b8b8').'" 
                    class="mrpu-color-field"
                    data-preview-target="text-color">';
    }

    public function font_size_callback() {
        $options = get_option('mrpu_display_options');
        echo '<input type="number" name="mrpu_display_options[font_size]" value="'.esc_attr($options['font_size'] ?? 14).'" min="10" max="24" step="1" class="small-text"> px';
    }

    public function border_radius_callback() {
        $options = get_option('mrpu_display_options');
        echo '<input type="number" name="mrpu_display_options[border_radius]" value="'.esc_attr($options['border_radius'] ?? 4).'" min="0" max="20" step="1" class="small-text"> px';
    }
    
    public function username_color_callback() {
        $options = get_option('mrpu_display_options');
        echo '<input type="text" name="mrpu_display_options[username_color]" 
                    value="'.esc_attr($options['username_color'] ?? '#b8b8b8').'" 
                    class="mrpu-color-field"
                    data-preview-target="username-color">';
    }
    
    public function username_font_size_callback() {
        $options = get_option('mrpu_display_options');
        echo '<input type="number" name="mrpu_display_options[username_font_size]" 
                    value="'.esc_attr($options['username_font_size'] ?? 14).'" 
                    min="10" max="24" step="1" class="small-text"> px';
    }
    
    public function username_font_weight_callback() {
        $options = get_option('mrpu_display_options');
        $current = $options['username_font_weight'] ?? 'normal';
        echo '<select name="mrpu_display_options[username_font_weight]" class="postform">';
        echo '<option value="normal" '.selected($current, 'normal', false).'> '. esc_html__('Normal', 'my-random-post-usernames').'</option>';
        echo '<option value="bold" '.selected($current, 'bold', false).'>' . esc_html__('Bold', 'my-random-post-usernames').'</option>';
        echo '<option value="lighter" '.selected($current, 'lighter', false).'>' . esc_html__('Lighter', 'my-random-post-usernames').'</option>';
        echo '</select>';
    }
    
    public function container_padding_callback() {
        $options = get_option('mrpu_display_options');
        echo '<input type="number" name="mrpu_display_options[container_padding]" 
                    value="'.esc_attr($options['container_padding'] ?? 10).'" 
                    min="0" max="50" step="1" class="small-text"> px';
        echo '<p class="description">' . esc_html__('Padding around the username container.', 'my-random-post-usernames').'</p>';
    }
}