<?php
/**
 * Plugin Name: My Random Post Usernames
 * Description: Privacy-First Content Attribution - Generate & Display Anonymous/Random Author Identifiers for Posts
 * Author: Mohamed C.
 * Version: 1.3
 * Text Domain: my-random-post-usernames
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 */

// Prevent direct access
if (!defined('ABSPATH')) {
    exit;
}

// 0. Include the admin class file
require_once plugin_dir_path(__FILE__) . 'admin/class-bulk-usernames-admin.php';
require_once plugin_dir_path(__FILE__) . 'admin/class-bulk-usernames-settings.php';

/**
 * 00. Enqueue js & frontend styles for displaying the username.
 */
function mrpu_enqueue_admin_assets( $hook ) {
    // Only load on our bulk usernames admin page.
    if ( 'tools_page_mrpu-bulk-usernames' !== $hook ) {
        return;
    }

    // Enqueue WordPress color picker styles.
    wp_enqueue_style('wp-color-picker');

    // Enqueue your admin CSS.
    wp_register_style(
        'mrpu-admin-style',
        plugins_url('css/mrpu-admin.css', __FILE__),
        array(),
        filemtime(plugin_dir_path(__FILE__) . 'css/mrpu-admin.css')
    );
    wp_enqueue_style('mrpu-admin-style');

    // Enqueue your admin JS and add 'wp-color-picker' as a dependency.
    wp_register_script(
        'mrpu-admin-script',
        plugins_url('js/mrpu-admin.js', __FILE__),
        array('jquery', 'wp-color-picker'), // ensure wp-color-picker is loaded!
        filemtime(plugin_dir_path(__FILE__) . 'js/mrpu-admin.js'),
        true
    );
    wp_enqueue_script('mrpu-admin-script');

    // Localize any needed variables.
    wp_localize_script('mrpu-admin-script', 'mrpu_admin_vars', array(
        'nonce_bulk' => wp_create_nonce('bulk_username_generation'),
        'nonce_edit' => wp_create_nonce('mrpu-edit-username'),
        'no_username'=> __('No username', 'my-random-post-usernames'),
        'ajaxurl' => admin_url('admin-ajax.php'),
        'resetNonce' => wp_create_nonce('mrpu-reset-nonce'),
        'resetConfirmation' => esc_html__('Are you sure you want to reset all random usernames? This cannot be undone.', 'my-random-post-usernames'),
        'resetSuccess' => esc_html__('Usernames have been reset.', 'my-random-post-usernames'),
        'resetError' => esc_html__('There was an error resetting the usernames.', 'my-random-post-usernames'),
    ));
}
add_action('admin_enqueue_scripts', 'mrpu_enqueue_admin_assets');

function mrpu_enqueue_frontend_assets() {
    wp_register_style(
        'mrpu-display-style',
        plugins_url('css/mrpu-display.css', __FILE__),
        array(),
        filemtime(plugin_dir_path(__FILE__) . 'css/mrpu-display.css')
    );
    wp_enqueue_style('mrpu-display-style');

    // If you need to add custom inline CSS from settings:
    $options = get_option('mrpu_display_options');
}
add_action('wp_enqueue_scripts', 'mrpu_enqueue_frontend_assets');


// -----------------------------------------
// 0. Add settings link to plugin actions
// -----------------------------------------
add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'mrpu_add_action_links');

function mrpu_add_action_links($links) {
    // Add "Bulk Usernames" link
    $settings_link = '<a href="' . admin_url('tools.php?page=mrpu-bulk-usernames') . '">' . __('Generate Usernames', 'my-random-post-usernames') . '</a>';
    
    // Add "Donate" link
    $donate_link = '<a href="https://www.paypal.com/donate/?hosted_button_id=7GYW2YYC6NHGJ" target="_blank" style="color: #3db634; font-weight: bold;">' . __('Donate', 'my-random-post-usernames') . '</a>';
    
    // Insert the links at the beginning of the array
    array_unshift($links, $settings_link, $donate_link);
    
    return $links;
}

// -----------------------------------------
// 1. Hooks for Admin Menu (Import the Class)
// -----------------------------------------
function mrpu_register_admin_page() {
    // Create an instance of our admin class
    $bulk_admin_page = new mrpu_Bulk_Usernames_Admin();
    $bulk_admin_page->init_hooks();
    
    $settings_page = new mrpu_Bulk_Usernames_Settings();
}
add_action('admin_menu', 'mrpu_register_admin_page');

// -------------------------------------------------------------
// 1. Meta Box to Show Random Username on Post Edit Screen
// -------------------------------------------------------------
function mrpu_add_random_username_meta_box() {
    add_meta_box(
        'random_username_meta_box',
        'Random Username',
        'mrpu_display_random_username_meta_box',
        'post',
        'side',
        'high'
    );
}
add_action('add_meta_boxes', 'mrpu_add_random_username_meta_box');

function mrpu_display_random_username_meta_box($post) {
    // Add nonce for security
    wp_nonce_field('random_username_meta_box', 'random_username_nonce');

    // Get existing username if it exists
    $random_username = get_post_meta($post->ID, '_random_username', true);

    // If no username exists, generate one
    if (empty($random_username)) {
        $random_username = mrpu_generate_random_username();
    }
    ?>
    <p>
        <label for="random_username">Username:</label>
        <input type="text" id="random_username" name="random_username" 
               value="<?php echo esc_attr($random_username); ?>" readonly>
        <button type="button" class="button" id="generate_new_username">Generate New</button>
    </p>
    <?php
}

function mrpu_save_random_username_meta($post_id) {
    // Check if nonce is set
    if (!isset($_POST['random_username_nonce'])) {
        return;
    }

    // Unsanitize and sanitize the nonce
    $nonce = sanitize_text_field(wp_unslash($_POST['random_username_nonce']));

    // Verify nonce
    if (!wp_verify_nonce($nonce, 'random_username_meta_box')) {
        return;
    }

    // If autosave, don't save
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
        return;
    }

    // Check user permissions
    if (!current_user_can('edit_post', $post_id)) {
        return;
    }

    // Save the username
    if (isset($_POST['random_username'])) {
        if (isset($_POST['random_username'])) {
            $random_username = sanitize_text_field(wp_unslash($_POST['random_username']));
            update_post_meta($post_id, '_random_username', $random_username);
        }
    }
}
add_action('save_post', 'mrpu_save_random_username_meta');

// -------------------------------------------------------------
// 2. Username Generation Function
// -------------------------------------------------------------
function mrpu_generate_random_username() {
    // Extensive lists for more variety
    $common_names = array(
        'james', 'john', 'robert', 'michael', 'william', 'david', 'richard', 'joseph', 'thomas', 'charles',
        'chris', 'daniel', 'matthew', 'anthony', 'donald', 'mark', 'paul', 'steven', 'andrew', 'kenneth',
        'josh', 'kevin', 'brian', 'george', 'tim', 'ron', 'jason', 'edward', 'jeff', 'ryan', 'jacob',
        'gary', 'nicholas', 'eric', 'jonathan', 'stephen', 'larry', 'justin', 'scott', 'brandon', 'ben',
        'samuel', 'gregory', 'alexander', 'patrick', 'frank', 'raymond', 'jack', 'dennis', 'jerry', 'tyler',
        'aaron', 'jose', 'adam', 'nathan', 'henry', 'douglas', 'peter', 'kyle', 'noah', 'ethan'
    );

    $common_words = array(
        'gaming', 'plays', 'tube', 'stream', 'live', 'videos', 'media', 'films', 'clips', 'content',
        'official', 'real', 'original', 'best', 'top', 'prime', 'elite', 'ultra', 'mega', 'super',
        'pro', 'master', 'expert', 'genius', 'guru', 'king', 'boss', 'chief', 'ninja', 'wizard',
        'beast', 'savage', 'legend', 'epic', 'amazing', 'cool', 'awesome', 'supreme', 'royal', 'premium',
        'gaming', 'gamer', 'player', 'streamer', 'creator', 'director', 'producer', 'editor', 'artist', 'designer'
    );

    $adjectives = array(
        'cool', 'epic', 'pro', 'dark', 'light', 'swift', 'quick', 'fast', 'smart', 'clever',
        'wild', 'crazy', 'mad', 'brave', 'bold', 'quiet', 'silent', 'loud', 'strong', 'mighty',
        'neat', 'pure', 'raw', 'rare', 'rich', 'safe', 'sharp', 'soft', 'solid', 'tight',
        'tough', 'urban', 'vital', 'wise', 'young', 'fresh', 'free', 'happy', 'rapid', 'prime'
    );

    $hobbies = array(
        'game', 'play', 'art', 'music', 'photo', 'video', 'film', 'code', 'tech', 'dev',
        'write', 'blog', 'vlog', 'speak', 'teach', 'learn', 'make', 'craft', 'build', 'mod',
        'mix', 'edit', 'shoot', 'stream', 'cast', 'record', 'create', 'design', 'style', 'train'
    );

    // More realistic username patterns
    $patterns = array(
        // Simple patterns
        function() use ($common_names) {
            return $common_names[array_rand($common_names)] . wp_rand(10, 9999);
        },
        // Gaming style
        function() use ($common_names, $common_words) {
            return $common_names[array_rand($common_names)] . 
                   $common_words[array_rand($common_words)] . 
                   wp_rand(10, 999);
        },
        // Professional style
        function() use ($common_words, $hobbies) {
            return $hobbies[array_rand($hobbies)] . 
                   $common_words[array_rand($common_words)] . 
                   wp_rand(100, 999);
        },
        // Trendy style with underscore
        function() use ($adjectives, $common_words) {
            return $adjectives[array_rand($adjectives)] . '_' . 
                   $common_words[array_rand($common_words)] . 
                   wp_rand(10, 99);
        },
        // Mixed style
        function() use ($common_names, $hobbies) {
            return $common_names[array_rand($common_names)] . 
                   $hobbies[array_rand($hobbies)] . 
                   wp_rand(100, 999);
        },
        // Short style
        function() use ($common_names, $adjectives) {
            return substr($common_names[array_rand($common_names)], 0, 1) . 
                   $adjectives[array_rand($adjectives)] . 
                   wp_rand(10, 999);
        },
        // Double word style
        function() use ($adjectives, $hobbies) {
            return $adjectives[array_rand($adjectives)] . 
                   $hobbies[array_rand($hobbies)] . 
                   wp_rand(10, 99);
        },
        // Year style
        function() use ($common_names) {
            $years = array('2000', '2001', '2002', '2003', '2004', '2005', '2006', '2007', '2008', '2009');
            return $common_names[array_rand($common_names)] . 
                   $years[array_rand($years)];
        },
        // xX style (popular in gaming)
        function() use ($common_names, $adjectives) {
            return 'xX' . $common_names[array_rand($common_names)] . 
                   $adjectives[array_rand($adjectives)] . 'Xx';
        },
        // Abbreviated style
        function() use ($common_words, $hobbies) {
            return strtoupper(substr($common_words[array_rand($common_words)], 0, 3)) . 
                   $hobbies[array_rand($hobbies)] . 
                   wp_rand(10, 99);
        }
    );
    
    // Select random pattern
    $pattern = $patterns[array_rand($patterns)];
    $username = strtolower($pattern());
    
    // Sometimes add special characters (20% chance)
    if (wp_rand(1, 100) <= 20) {
        $special_chars = array('.', '_', '-');
        $char = $special_chars[array_rand($special_chars)];
        $username = str_replace(' ', $char, $username);
    }
    
    // Sometimes add l33t speak (10% chance)
    if (wp_rand(1, 100) <= 10) {
        $leetReplacements = array(
            'a' => '4',
            'e' => '3',
            'i' => '1',
            'o' => '0',
            's' => '5',
            't' => '7'
        );
        $username = strtr($username, $leetReplacements);
    }
    
    return $username;
}

// -------------------------------------------------------------
// 3. Display on Frontend (Updated with Settings)
// -------------------------------------------------------------
function mrpu_display_random_username($content) {
    global $wpdb;
    if (is_single()) {
        $options = get_option('mrpu_display_options');
        
        // Use get_post_meta instead of direct DB query
        $random_username = get_post_meta(get_the_ID(), '_random_username', true);

        if (!empty($random_username)) {
            // Add your logic to display the username
            //$content .= '<div class="random-username">' . esc_html($random_username) . '</div>';
        }
        
        if (!empty($random_username)) {
            // Get all style values
            $container_style = sprintf(
                'background: %s; 
                border-radius: %dpx; 
                font-size: %dpx; 
                padding: %dpx; ',
                esc_attr($options['background_color'] ?? '#1b1b1b'),
                esc_attr($options['border_radius'] ?? 4),
                esc_attr($options['font_size'] ?? 14),
                esc_attr($options['container_padding'] ?? 10)
            );
            
            $username_style = sprintf(
                'color: %s; font-size: %dpx; font-weight: %s;',
                esc_attr($options['username_color'] ?? '#b8b8b8'),
                esc_attr($options['username_font_size'] ?? 14),
                esc_attr($options['username_font_weight'] ?? 'normal')
            );

            // Build the HTML
            $username_html = sprintf(
                '<div class="mrpu-uploader" style="%s">
                    <span class="mrpu-posted-by" style="color: %s">%s </span>
                    <span class="mrpu-username" style="%s">%s</span>
                </div>',
                $container_style,
                esc_attr($options['text_color'] ?? '#b8b8b8'),
                esc_html($options['custom_text'] ?? __('Uploaded by', 'my-random-post-usernames')),
                $username_style,
                esc_html($random_username)
            );
            
            $content = $username_html . $content;
        }
    }
    return $content;
}
add_filter('the_content', 'mrpu_display_random_username');

function mrpu_add_random_username_styles() {
    if (is_single()) {
        $options = get_option('mrpu_display_options');
    }
}
add_action('wp_head', 'mrpu_add_random_username_styles');

// -------------------------------------------------------------
// 4. AJAX: Generate New Username (Individual)
// -------------------------------------------------------------
function mrpu_ajax_generate_new_username() {
    echo esc_html( mrpu_generate_random_username() );
    wp_die();
}
add_action('wp_ajax_generate_new_username', 'mrpu_ajax_generate_new_username');


// -------------------------------------------------------------
// 6. AJAX Handler for Bulk Processing (New or Existing)
// -------------------------------------------------------------

/**
 * Optionally clear caches if your site or hosting environment
 * uses object caching or transients heavily.
 */
function mrpu_clear_username_caches() {
    // Clear post meta cache for all posts
    $posts = get_posts(array(
        'post_type'      => 'any', // Include all post types
        'posts_per_page' => -1,   // Get all posts
        'fields'         => 'ids' // Only retrieve post IDs
    ));

    // Loop through posts and clear their meta cache
    foreach ($posts as $post_id) {
        wp_cache_delete($post_id, 'post_meta');
    }

    // Clear all transients (optional, but more efficient than raw SQL)
    if (function_exists('wp_delete_all_transients')) {
        wp_delete_all_transients();
    }

    // Clear object cache
    wp_cache_flush();
}

function mrpu_process_username_batch() {
    global $wpdb;
    check_ajax_referer('bulk_username_generation', 'nonce');

    if (!current_user_can('manage_options')) {
        wp_send_json_error(__('Unauthorized', 'my-random-post-usernames'));
        return;
    }
    
    $batch_size      = isset($_POST['batch_size']) ? intval($_POST['batch_size']) : 20;
    $processed_so_far = isset($_POST['processed_so_far']) ? intval($_POST['processed_so_far']) : 0;
    $regenerate = isset($_POST['regenerate']) ? filter_var(wp_unslash($_POST['regenerate']), FILTER_VALIDATE_BOOLEAN) : false;
    
    // For missing posts, always use OFFSET 0.
    $offset = $regenerate ? $processed_so_far : 0;
    
    $logs = array();
    
    if ($regenerate) {
        // Get posts that already have a random username
        $query = new WP_Query(array(
            'post_type'      => 'post',
            'post_status'    => 'publish',
            'posts_per_page' => $batch_size,
            'offset'         => $offset,
            'meta_key'       => '_random_username',
            'fields'         => 'ids', // Only retrieve post IDs
        ));
        $post_ids = $query->posts;
    } else {
        // Get posts that do not have a random username
        $query = new WP_Query(array(
            'post_type'      => 'post',
            'post_status'    => 'publish',
            'posts_per_page' => $batch_size,
            'offset'         => $offset,
            'meta_query'     => array(
                array(
                    'key'     => '_random_username',
                    'compare' => 'NOT EXISTS',
                ),
            ),
            'fields'         => 'ids', // Only retrieve post IDs
        ));
        $post_ids = $query->posts;
    }
    
    $processed = 0;
    
    foreach ($post_ids as $post_id) {
        $new_username = mrpu_generate_random_username();
        
        if ($regenerate) {
            // Update existing random username
            update_post_meta($post_id, '_random_username', $new_username);
        } else {
            // Add new random username
            add_post_meta($post_id, '_random_username', $new_username);
        }
        
        // Get the post title (if available) and log the action.
        $post_title = get_the_title($post_id);
        
        // Translators: %1$s is the post title, %2$d is the post ID.
        $logs[] = sprintf(__('Generated username for post "%1$s" (ID: %2$d)', 'my-random-post-usernames'),$post_title,$post_id);
        
        $processed++;
    }

    $total_processed = $processed_so_far + $processed;

    // Optionally clear caches
    mrpu_clear_username_caches();

    wp_send_json_success(array(
        'total_processed' => $total_processed,
        'batch_processed' => $processed,
        'logs'            => $logs
    ));
}
add_action('wp_ajax_process_username_batch', 'mrpu_process_username_batch');

// AJAX handler
function mrpu_save_username_callback() {
    // Verify the nonce (ensure the nonce key 'security' matches what your JS sends)
    check_ajax_referer('mrpu-edit-username', 'security');

    // Check if 'post_id' is set
    if (!isset($_POST['post_id'])) {
        wp_send_json_error(__('Missing post ID.', 'my-random-post-usernames'));
    }

    // Sanitize and validate the post ID
    $post_id = absint(wp_unslash($_POST['post_id']));
    if (!$post_id || !current_user_can('edit_post', $post_id)) {
        wp_send_json_error(__('Unauthorized or invalid post ID.', 'my-random-post-usernames'));
    }

    // Check if 'username' is set
    if (isset($_POST['username'])) {
        // Remove slashes and sanitize the username
        $username = sanitize_text_field(wp_unslash($_POST['username']));

        if (empty($username)) {
            delete_post_meta($post_id, '_random_username');
        } else {
            update_post_meta($post_id, '_random_username', $username);
        }
    } else {
        wp_send_json_error(__('Username not provided.', 'my-random-post-usernames'));
    }

    wp_send_json_success();
}
add_action('wp_ajax_mrpu_save_username', 'mrpu_save_username_callback');

// Localization strings
add_action('admin_init', function() {
    wp_localize_script('jquery', 'mrpu_vars', [
        'no_username' => __('No username', 'my-random-post-usernames')
    ]);
});


function mrpu_ajax_reset_usernames() {
    check_ajax_referer('mrpu-reset-nonce', 'nonce');
    
    if ( ! current_user_can('manage_options') ) {
        wp_send_json_error(__('Unauthorized', 'my-random-post-usernames'));
    }
    
    global $wpdb;
    // Delete all random usernames from post meta
    delete_post_meta_by_key('_random_username');
    
    // Clear caches if needed
    wp_cache_flush();
    
    wp_send_json_success(__('All random usernames have been reset.', 'my-random-post-usernames'));
}
add_action('wp_ajax_reset_usernames', 'mrpu_ajax_reset_usernames');